@@ -39,23 +39,26 @@ static size_t print_httppost_callback(void *arg, const char *buf, size_t len)
     }                                                            \
   } while(0)
 
-static CURLcode test_lib1308(char *URL)
+static CURLcode test_lib1308(const char *URL)
 {
   int errorcount = 0;
   CURLFORMcode rc;
-  int res;
+  CURLcode res = CURLE_OK;
+  int formres = 0;
   struct curl_httppost *post = NULL;
   struct curl_httppost *last = NULL;
   size_t total_size = 0;
   char buffer[] = "test buffer";
 
+  global_init(CURL_GLOBAL_ALL);
+
   rc = curl_formadd(&post, &last, CURLFORM_COPYNAME, "name",
                     CURLFORM_COPYCONTENTS, "content", CURLFORM_END);
   t1308_fail_unless(rc == 0, "curl_formadd returned error");
 
-  /* after the first curl_formadd when there's a single entry, both pointers
+  /* after the first curl_formadd when there is a single entry, both pointers
      should point to the same struct */
-  t1308_fail_unless(post == last, "post and last weren't the same");
+  t1308_fail_unless(post == last, "post and last were not the same");
 
   rc = curl_formadd(&post, &last, CURLFORM_COPYNAME, "htmlcode",
                     CURLFORM_COPYCONTENTS, "<HTML></HTML>",
@@ -66,8 +69,8 @@ static CURLcode test_lib1308(char *URL)
                     CURLFORM_PTRCONTENTS, buffer, CURLFORM_END);
   t1308_fail_unless(rc == 0, "curl_formadd returned error");
 
-  res = curl_formget(post, &total_size, print_httppost_callback);
-  t1308_fail_unless(res == 0, "curl_formget returned error");
+  formres = curl_formget(post, &total_size, print_httppost_callback);
+  t1308_fail_unless(formres == 0, "curl_formget returned error");
 
   t1308_fail_unless(total_size == 518, "curl_formget got wrong size back");
 
@@ -83,12 +86,14 @@ static CURLcode test_lib1308(char *URL)
                     CURLFORM_END);
   t1308_fail_unless(rc == 0, "curl_formadd returned error");
 
-  res = curl_formget(post, &total_size, print_httppost_callback);
+  formres = curl_formget(post, &total_size, print_httppost_callback);
 
-  t1308_fail_unless(res == 0, "curl_formget returned error");
+  t1308_fail_unless(formres == 0, "curl_formget returned error");
   t1308_fail_unless(total_size == 899, "curl_formget got wrong size back");
 
   curl_formfree(post);
 
+  curl_global_cleanup();
+
   return errorcount ? TEST_ERR_FAILURE : CURLE_OK;
 }
